/*
   3D Invaders

   player.cc

   (c)2002 Paul Boese
*/

#include "common.h"
#include "player.h"
#include "ship_data.h"

#define EXPLODE_FRMS 30

enum { P_ACTIVE, P_SHIELD, P_HIT, P_EXPLODE, P_GONE };

Player::Player() {
	mode = P_ACTIVE;
	paused = false;
	explode = 0;
	position.x = 0.0f;
	position.y = 0.0f;
	position.z = 5.0f;
}

void Player::Reset(point_t * pos) {
	mode = P_ACTIVE;
	paused = false;
	explode = 0;
	position.x = pos->x;
	position.y = pos->y;
	position.z = pos->z;
}

void Player::draw_ship() {
	int i;
	glEnable(GL_TEXTURE_2D);
	glBindTexture( GL_TEXTURE_2D, txship);
	glTexEnvi(GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_MODULATEALPHA);
	glTranslatef(0.0f, 0.0f, 1.0f);
	glRotatef(90.0f, 0.0f, 1.0f, 0.0f);
	glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
	glScalef(0.5f, 0.5f, 0.5f);
	glBegin(GL_TRIANGLES);
	/* Faces */
	for (i=0; i<SHIP_FACE_IDXS; i+=4) {
		/* Note this is reading the face indexs backwards
		   so that the face vertices are output in clockwise
		   order. */
		glTexCoord2f(
			ship_uv[i+2][0], 
			ship_uv[i+2][1]);
		glVertex3f(
			ship_coords[ship_face_idx[i+2]][0],
			ship_coords[ship_face_idx[i+2]][1],
			ship_coords[ship_face_idx[i+2]][2]);
		
		glTexCoord2f(
			ship_uv[i+1][0],
			ship_uv[i+1][1]);
		glVertex3f(
			ship_coords[ship_face_idx[i+1]][0],
			ship_coords[ship_face_idx[i+1]][1],
			ship_coords[ship_face_idx[i+1]][2]);
		
		glTexCoord2f(
			ship_uv[i][0],
			ship_uv[i][1]);
		glVertex3f(
			ship_coords[ship_face_idx[i]][0],
			ship_coords[ship_face_idx[i]][1],
			ship_coords[ship_face_idx[i]][2]);
		}
	glEnd();
}

void Player::draw_player() {
        glPushMatrix();
        glTranslatef(position.x, position.y, position.z);
	glColor4f(1.0f, 1.0f, 1.0f, 0.2f);
	draw_ship();
        glPopMatrix();
}

int Player::draw_explosion() {
	if (((explode%4) < 2)) {
        	glPushMatrix();
	        glTranslatef(position.x, position.y, position.z);
        	glColor4f(1.0f, 0.5f, 0.5f, 0.2f);
		draw_ship();
	        glPopMatrix();
	}
	if (--explode > 0)
		return P_EXPLODE;
	else
		return P_ACTIVE;
	
};

void Player::Up() {
	if (position.y > -8.0f && !paused)
		position.y += -0.15f;
}

void Player::Down() {
	if (position.y < 8.0f && !paused)
		position.y += 0.15;
}

void Player::Right() {
	if (position.x > -8.0f && !paused)
		position.x += -0.15f;
}

void Player::Left() {
	if (position.x < 8.0f && !paused)
		position.x += 0.15;
}

void Player::Hit() {
	mode = P_HIT;
}

void Player::Pause() {
	if (!paused) {
		paused = true;
	}
}

void Player::Resume() {
	if (paused) {
		paused = false;
	}
}

void Player::Draw() {
	switch(mode) {
	case P_ACTIVE:
		draw_player();
		break;
	case P_SHIELD:
		draw_player();
		break;
	case P_HIT:
		snd_sfx_play(sfx_player_hit, gbl_sfx_volume, 128); 
		mode = P_EXPLODE;
		explode = EXPLODE_FRMS;
	case P_EXPLODE:
		mode = draw_explosion();
		break;
	case P_GONE:
		break;
	}
}
